package com.ims.eval.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.common.utils.Constant;
import com.ims.eval.cache.CacheContext;
import com.ims.eval.config.CustomException;
import com.ims.eval.dao.ResponsibilityIndicatorInfoMapper;
import com.ims.eval.entity.Indicator;
import com.ims.eval.entity.OrganizationEvaluationInfo;
import com.ims.eval.entity.ResponsibilityIndicatorInfo;
import com.ims.eval.entity.dto.request.ResponsibilityIndicatorInfoUpdateDTO;
import com.ims.eval.entity.dto.response.IndicatorResDTO;
import com.ims.eval.entity.dto.response.MyuserResDTO;
import com.ims.eval.entity.dto.response.OrganizationEvaluationInfoResDTO;
import com.ims.eval.entity.dto.response.ResponsibilityIndicatorInfoResDTO;
import com.ims.eval.service.IIndicatorService;
import com.ims.eval.service.IResponsibilityIndicatorInfoService;
import com.ims.eval.service.IUserService;
import com.ims.eval.util.MathCalculatorUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 责任书指标明细
 * <p>
 * 服务实现类
 * </p>
 *
 * @author wang
 * @since 2023-03-08
 */
@Service
public class ResponsibilityIndicatorInfoServiceImpl extends ServiceImpl<ResponsibilityIndicatorInfoMapper, ResponsibilityIndicatorInfo> implements IResponsibilityIndicatorInfoService {


	@Autowired
	private IUserService userService;

	@Autowired
	private IIndicatorService indicatorService;

	@Override
	public Map<Object, List<ResponsibilityIndicatorInfoResDTO>> planValueList(List<String> deptResponsibilityIds, String dept, HttpServletRequest request) {

		List<ResponsibilityIndicatorInfoResDTO> list = baseMapper.listByresponsibilityId(deptResponsibilityIds, dept, "ZRMB", "");//默认获取指定的怎目标

		Map<Object, List<ResponsibilityIndicatorInfoResDTO>> map = list.stream().collect(Collectors.groupingBy(ResponsibilityIndicatorInfoResDTO::getIsQuantified));
		map.put("quantifiedList", map.remove(true));
		map.put("nonQuantifiedList", map.remove(false));
		return map;

	}


	@Override
	public Map<Object, List<ResponsibilityIndicatorInfoResDTO>> selectResponsibilityIndicatorInfoList(String deptResponsibilityId, String isQuantified, String dataState) {
		List<ResponsibilityIndicatorInfoResDTO> list = baseMapper.selectResponsibilityIndicatorInfoList(deptResponsibilityId, isQuantified, dataState);
		Map<Object, List<ResponsibilityIndicatorInfoResDTO>> map = list.stream().collect(Collectors.groupingBy(ResponsibilityIndicatorInfoResDTO::getIsQuantified));
		map.put("quantifiedList", map.remove(true));
		map.put("nonQuantifiedList", map.remove(false));
		return map;
	}


	@Transactional
	@Override
	public boolean saveOrUpdateDto(ResponsibilityIndicatorInfoUpdateDTO infoUpdateDTO) {
		ResponsibilityIndicatorInfo indicatorInfo = baseMapper.selectById(infoUpdateDTO.getId());
		indicatorInfo.setQuantifiedValue(infoUpdateDTO.getQuantifiedValue());//量化值
		indicatorInfo.setNonQuantifiedValue(infoUpdateDTO.getNonQuantifiedValue());//非量化值
		indicatorInfo.setUpdateTime(new Date());//更新时间
		indicatorInfo.setRemark(infoUpdateDTO.getRemark());//dicatorInfo.setUpdateBy(infoUpdateDTO.getUpdateBy());//更新者备注
		return super.saveOrUpdate(indicatorInfo);
	}


	@Transactional
	@Override
	public boolean saveBatchDto(List<ResponsibilityIndicatorInfoUpdateDTO> infoUpdateDTOs) {
		for (ResponsibilityIndicatorInfoUpdateDTO entity : infoUpdateDTOs) {

			if (null != entity && null != entity.getId() && !"".equals(entity.getId().trim())) {
				ResponsibilityIndicatorInfo indicatorInfo = baseMapper.selectById(entity.getId());

				indicatorInfo.setQuantifiedValue(entity.getQuantifiedValue());//量化值
				indicatorInfo.setNonQuantifiedValue(entity.getNonQuantifiedValue());//非量化值
				indicatorInfo.setUpdateTime(new Date());//更新时间
				indicatorInfo.setUpdateBy(entity.getUpdateBy());//更新者
				indicatorInfo.setRemark(entity.getRemark());//备注
				indicatorInfo.setState(entity.getState());//状态
				boolean b = super.saveOrUpdate(indicatorInfo);
				if (!b) {
					throw new CustomException("更新失败");
				}
			} else {
				ResponsibilityIndicatorInfo addInfo = new ResponsibilityIndicatorInfo();
				addInfo.setDeptResponsibilityId(entity.getDeptResponsibilityId());
				addInfo.setOrganizationEvaluationRuleId(entity.getOrganizationEvaluationRuleId());
				addInfo.setIndicatorId(entity.getIndicatorId());
//				addInfo.setIndicatorDictionaryId(entity.getIndicatorDictionaryId());
				addInfo.setOptionCode(entity.getOptionCode());
				addInfo.setIsQuantified(entity.getIsQuantified());
				addInfo.setQuantifiedValue(entity.getQuantifiedValue());
				addInfo.setNonQuantifiedValue(entity.getNonQuantifiedValue());
				addInfo.setRemark(entity.getRemark());//备注
				addInfo.setCreateTime(new Date());
				addInfo.setState("-1");//状态

				Indicator indicator = indicatorService.getById(entity.getIndicatorId());
				addInfo.setDeptId(indicator.getDeptId());

				boolean b = super.saveOrUpdate(addInfo);
				if (!b) {
					throw new CustomException("新增失败");
				}
			}

		}
		return true;
	}

	@Override
	public Map<String, Object> getResponsibilityIdMap(String deptResponsibilityId) {
		Map<String, Object> map = new HashMap<>();
		QueryWrapper<ResponsibilityIndicatorInfo> qw = new QueryWrapper<>();
		qw.lambda().eq(ResponsibilityIndicatorInfo::getDeptResponsibilityId, deptResponsibilityId);
		List<ResponsibilityIndicatorInfo> list = baseMapper.selectList(qw);
		list.stream().forEach(i -> {
			map.put(i.getIndicatorId(), i.getIsQuantified().equals("是") ? i.getQuantifiedValue() : i.getQuantifiedValue());
		});
		return map;
	}

	@Override
	public boolean saveOrUpdateBatch2(List<ResponsibilityIndicatorInfo> responsibilityIndicatorInfoList) {
		boolean b = false;
		for (ResponsibilityIndicatorInfo obj : responsibilityIndicatorInfoList) {
			ResponsibilityIndicatorInfo responsibilityIndicatorInfo = super.getById(obj.getId());
			if (!"失效".equals(responsibilityIndicatorInfo.getDataState())) {
				obj.setDataState("失效");
				b = super.updateById(obj);
				if (b) {
					ResponsibilityIndicatorInfo info = new ResponsibilityIndicatorInfo();
					info.setDeptResponsibilityId(responsibilityIndicatorInfo.getDeptResponsibilityId());
					info.setOrganizationEvaluationRuleId(responsibilityIndicatorInfo.getOrganizationEvaluationRuleId());
					info.setIndicatorId(responsibilityIndicatorInfo.getIndicatorId());
					info.setIndicatorDictionaryId(responsibilityIndicatorInfo.getIndicatorDictionaryId());
					info.setOptionCode(responsibilityIndicatorInfo.getOptionCode());
					info.setIsQuantified(obj.getIsQuantified());
					info.setQuantifiedValue(obj.getQuantifiedValue());
					info.setNonQuantifiedValue(obj.getNonQuantifiedValue());
					info.setCreateTime(new Date());
					info.setState(obj.getState());
					info.setDataState("有效");
					b = super.save(info);
				}
			} else {
				obj.setUpdateTime(new Date());
				b = super.updateById(obj);
			}
		}
		return b;
	}

	@Override
	public List<ResponsibilityIndicatorInfoResDTO> selectPopUpData(String deptResponsibilityId, String isQuantified, String dataState) {
		List<ResponsibilityIndicatorInfoResDTO> list = baseMapper.selectResponsibilityIndicatorInfoList(deptResponsibilityId, isQuantified, dataState);
		return list;
	}


	@Override
	public List<Map> getResponsibilityIndicatorList(String responsibilityId, String binSection, String binStage, HttpServletRequest request) {

		//判断是否是（重点专项和管理事项）
		List<Map> indicatorMap = new ArrayList<>();
		if("ZDZX".equals(CacheContext.bseIdObject.get(binStage).getStageCode()) || "GLSX".equals(CacheContext.bseIdObject.get(binStage).getStageCode())){

			List<Indicator> indicatorList = indicatorService.listAll("","","",binSection,binStage,"","");
			if(null != indicatorList && indicatorList.size()>0){
				for (Indicator i : indicatorList){
					Map map = new HashMap();
					map.put("id",i.getId());
					map.put("indicator_name",i.getIndicatorName());
					indicatorMap.add(map);
				}
			}

		}else{
			indicatorMap = baseMapper.selectResponsibilityIndicatorList(responsibilityId, binSection, binStage);
		}

		return indicatorMap;
	}

	@Override
	public Map getResponsibilityInfoList(String responsibilityId, String indicatorId, String organizationShortName, String binSection, String binStage, HttpServletRequest request) {
		Map data = new HashMap();
		//设置数据头
		//获取指标名称

		boolean titlemark = true;//标记生成标题

		List<Map> indicatorMap  = getResponsibilityIndicatorList(responsibilityId, binSection, binStage,request);

		List<String> idList = indicatorMap.stream()
			.map(map -> String.valueOf(map.get("id"))) // 提取每个Map中key为"id"的值，并转换成String
			.collect(Collectors.toList()); // 将提取的值收集到List中
		if(null == idList || idList.size()<=0){
				throw  new CustomException("未查询导数据");
		}
		List<IndicatorResDTO> indicatorResDTOS = indicatorService.getGroupChildCodeByIds(idList);

		//根据指标id和指标名分组
		Map<String, List<IndicatorResDTO>> groupedMap = indicatorResDTOS.stream()
			.collect(Collectors.groupingBy(dto -> dto.getIndicatorId() + "," + dto.getIndicatorName())); // 根据indicatorId和indicatorName进行分组

		if (titlemark) {
			// 遍历分组后的Map
			Map title = new LinkedHashMap();//保存所有的指标名
			for (Map.Entry<String, List<IndicatorResDTO>> entry : groupedMap.entrySet()) {

				List<IndicatorResDTO> groupedList = entry.getValue(); // 获取分组后的List
				if(null == groupedList || groupedList.size()<=0){
					continue;
				}
				// 进一步处理每个分组
				List<Map> titleArray = new ArrayList<>();
				for (IndicatorResDTO dto : groupedList) {
					// 处理每个分组中的元素
					Map titlemap = new LinkedHashMap();
					titlemap.put("childName",dto.getChildName());//子指标名
					titlemap.put("key", dto.getOptionName());//名称
					titlemap.put("code",dto.getChildCode()+"_"+dto.getOptionCode());//名称
					titlemap.put("flag", dto.getIsQuantified());//是否量化
					titleArray.add(titlemap);
				}
				title.put(entry.getKey().split(",")[1], titleArray);
			}

			data.put("title",title);
			titlemark = false;
		}


		//获取数据
		List<Map> datamapList = new ArrayList<>();
		List<ResponsibilityIndicatorInfoResDTO> dataList = baseMapper.getResponsibilityInfoList(responsibilityId, indicatorId,organizationShortName, binSection, binStage);

		Map<String, List<ResponsibilityIndicatorInfoResDTO>> groupeddataMap = dataList.stream().collect(Collectors.groupingBy(ResponsibilityIndicatorInfoResDTO::getOrganizationId)); // 根据organizationId进行分组

		// 遍历分组后的Map
		for (Map.Entry<String, List<ResponsibilityIndicatorInfoResDTO>> entry : groupeddataMap.entrySet()) {
			List<ResponsibilityIndicatorInfoResDTO> groupedList = entry.getValue(); // 获取分组后的List


			Map<String, ResponsibilityIndicatorInfoResDTO> resultMap = groupedList.stream()
				.collect(Collectors.toMap(
					dto -> dto.getChildCode() + "_" + dto.getOptionCode(),
					dto -> dto,
					(oldValue, newValue) -> oldValue)); // 解决键重复的情况


			Map indicatormap = new HashMap();
			// 进一步处理每个分组
			for (IndicatorResDTO d : indicatorResDTOS) {
				boolean mark = true;//标记给公司名赋值
				if (mark) {
					indicatormap.put("organizationShortName", resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getOrganizationShortName());//公司名
					indicatormap.put("organizationId", resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getOrganizationId());//公司id
					indicatormap.put("state",resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getState());
					indicatormap.put("IS_LH_state","3");
					mark = false;
				}
				//指标属性
				indicatormap.put(d.getChildCode() + "_" + d.getOptionCode(), null == resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getNonQuantifiedValue() ? "" : resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getNonQuantifiedValue());
				//指标id
				indicatormap.put("ID_" + d.getChildCode() + "_" + d.getOptionCode(), resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getId());
				//是否量化
				indicatormap.put("IS_LH_" +d.getChildCode() + "_" + d.getOptionCode(),resultMap.get(d.getChildCode()+"_"+d.getOptionCode()).getIsQuantified()?"1":"2");
			}
			datamapList.add(indicatormap);
		}

		data.put("value", datamapList);
		return data;
	}


	@Override
	public boolean updateResponsibilityIndicatorInfo(List<JSONObject> jsonObjects) {

		for (JSONObject json :jsonObjects){

			Map<String,Object > idMap = new HashMap();
			Map valueMap =  new HashMap();
			for (String key : json.keySet()) {
				Object value = json.get(key);
				if(key.startsWith("ID_")){
					idMap.put(key,value);
				}else {
					valueMap.put(key,value);
				}
			}

			List<ResponsibilityIndicatorInfo> responsibilityInfos =new ArrayList<>();
			for (Map.Entry<String,Object > entry : idMap.entrySet()) {
				ResponsibilityIndicatorInfo info = new ResponsibilityIndicatorInfo();
				info.setId(entry.getValue().toString());
				String quantified = valueMap.get(entry.getKey().replace("ID_","IS_LH_")).toString();
				if ("1".equals(quantified)) {

					if (MathCalculatorUtil.isNumber(String.valueOf(valueMap.get(entry.getKey().replace("ID_", "")).toString()))) {
						info.setQuantifiedValue(Double.valueOf(valueMap.get(entry.getKey().replace("ID_", "")).toString()));
					} else {
						info.setQuantifiedValue(0);
					}
				}

				info.setNonQuantifiedValue(valueMap.get(entry.getKey().replace("ID_","")).toString());
				info.setState(valueMap.get("state").toString());//
				responsibilityInfos.add(info);
			}
			return  this.saveOrUpdateBatch(responsibilityInfos);
		}

		return false;
	}
}
